/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <efl_extension.h>
#include "$(appName).h"
#include "view.h"
#include "view_common.h"

static struct view_info {
	Evas_Object *win;
	Evas_Object *conform;
	Evas_Object *navi;
	Evas_Object *list;
} s_info = {
	.win = NULL,
	.conform = NULL,
	.navi = NULL,
	.list = NULL,
};

static void _delete_win_request_cb(void *data, Evas_Object *obj, void *event_info);
static void _navi_back_cb(void *data, Evas_Object *obj, void *event_info);
static void _list_item_selected_cb(void *data, Evas_Object *obj, void *event_info);
static void _populate_list(void);

/**
 * @brief Creates essential objects: window, conformant and layout.
 */
Eina_Bool view_create(void *user_data)
{
	/* Create the window */
	s_info.win = view_create_win(PACKAGE);
	if (s_info.win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a window.");
		return EINA_FALSE;
	}

	/* Create the conformant */
	s_info.conform = view_create_conformant(s_info.win);
	if (s_info.conform == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a conformant");
		return EINA_FALSE;
	}

	/* Create the background */
	if (!view_create_background(s_info.conform, "elm.swallow.indicator_bg"))
		return EINA_FALSE;

	/* Create the naviframe for list */
	s_info.navi = view_create_naviframe(s_info.conform, "elm.swallow.content");
	if (!s_info.navi)
		return EINA_FALSE;

	/* Create the list */
	s_info.list = view_create_list(s_info.navi);
	if (!s_info.list)
		return EINA_FALSE;

	/* Adds items to the list */
	_populate_list();

	/* Push the list to naviframe */
	elm_naviframe_item_push(s_info.navi, "System Info", NULL, NULL, s_info.list, NULL);

	/* Show the window after main view is set up */
	evas_object_show(s_info.win);

	return EINA_TRUE;
}

/**
 * @brief Creates a basic window named package.
 * @param[in] pkg_name Name of the window.
 * @return This function return window object if it was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_win(const char *pkg_name)
{
	Evas_Object *win = NULL;

	/*
	 * Window
	 * Create and initialize elm_win.
	 * elm_win is mandatory to manipulate the window.
	 */
	win = elm_win_util_standard_add(pkg_name, pkg_name);
	elm_win_conformant_set(win, EINA_TRUE);
	elm_win_indicator_mode_set(win, ELM_WIN_INDICATOR_SHOW);
	elm_win_indicator_opacity_set(win, ELM_WIN_INDICATOR_OPAQUE);
	elm_win_autodel_set(win, EINA_TRUE);

	evas_object_smart_callback_add(win, "delete,request", _delete_win_request_cb, NULL);

	return win;
}

/**
 * @brief Creates a conformant object.
 * @param[in] win The object to which you want to set this conformant.
 * Conformant is mandatory for base GUI to have proper size.
 * @return This function return conformant object if it was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_conformant(Evas_Object *win)
{
	/*
	 * Conformant
	 * Create and initialize elm_conformant.
	 * elm_conformant is mandatory for base GUI to have proper size
	 * when indicator or virtual keypad is visible.
	 */
	Evas_Object *conform = NULL;

	if (win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "window is NULL.");
		return NULL;
	}

	conform = elm_conformant_add(win);
	evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, conform);

	evas_object_show(conform);

	return conform;
}

/**
 * @brief Creates a background for parent object and sets it to designated part.
 * @param[in] parent The parent object for background.
 * @param[in] part_name The name of the part within parent object.
 * where the background is to be set,
 * @return This function returns a background object if is was successfully created,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_background(Evas_Object *parent, const char *part_name)
{
	Evas_Object *bg = elm_bg_add(parent);
	if (!bg) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_bg_add() failed.");
		return NULL;
	}

	elm_object_style_set(bg, "indicator/headerbg");
	elm_object_part_content_set(parent, part_name, bg);
	evas_object_show(bg);

	return bg;
}

/**
 * @brief Creates a naviframe for parent object and sets it to designated part.
 * @param[in] parent The parent object for naviframe.
 * @param[in] part_name The name of the part within parent object
 * where the naviframe is to be set,
 * @return This function returns a naviframe object if is was successfully created,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_naviframe(Evas_Object *parent, const char *part_name)
{
	Evas_Object *navi = elm_naviframe_add(parent);
	if (!navi) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_naviframe_add() failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(navi, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_naviframe_prev_btn_auto_pushed_set(navi, EINA_FALSE);
	elm_object_part_content_set(s_info.conform, part_name, navi);
	eext_object_event_callback_add(navi, EEXT_CALLBACK_BACK, _navi_back_cb, NULL);

	evas_object_show(s_info.navi);

	return navi;
}

/**
 * @brief Creates a list for parent object.
 * @param[in] parent The parent object for a list.
 * @return This function returns a list object if is was successfully created,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_list(Evas_Object *parent)
{
	Evas_Object *list = elm_list_add(parent);
	if (!list) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_list_add() failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(list, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(list, EVAS_HINT_FILL, EVAS_HINT_FILL);

	elm_list_go(list);
	evas_object_show(list);

	return list;
}

/**
 * @brief Destroys window and frees its resources.
 */
void view_destroy(void)
{
	if (s_info.win == NULL)
		return;

	evas_object_del(s_info.win);
}

/**
 * @brief Internal callback function invoked when the main window needs to be destroyed.
 * @param[in] data The user data passed to the evas_object_smart_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _delete_win_request_cb(void *data, Evas_Object *obj, void *event_info)
{
	ui_app_exit();
}

/**
 * @brief Internal callback function invoked on HW Back button press.
 * @param[in] data The user data passed to the eext_object_event_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _navi_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	if (elm_naviframe_top_item_get(s_info.navi) == elm_naviframe_bottom_item_get(s_info.navi))
		elm_win_lower(s_info.win);
	else
		elm_naviframe_item_pop(s_info.navi);
}

/**
 * @brief Internal callback function invoked on list item selection.
 * @param[in] data The user data passed to the elm_list_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _list_item_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Smart_Cb on_select_cb = (Evas_Smart_Cb)data;
	Elm_Object_Item *item  = elm_list_selected_item_get(obj);

	elm_list_item_selected_set(item, EINA_FALSE);

	if (!on_select_cb) {
		dlog_print(DLOG_ERROR, LOG_TAG, "List item selection callback function is not attached.");
		return;
	}

	on_select_cb((void *)s_info.navi, obj, event_info);
}

/**
 * @brief Internal function which populates the list with main items.
 */
static void _populate_list(void)
{
	elm_list_item_append(s_info.list, "Camera", NULL, NULL, _list_item_selected_cb, (void *)view_camera_list_cb);
	elm_list_item_append(s_info.list, "Input", NULL, NULL, _list_item_selected_cb, (void *)view_input_list_cb);
	elm_list_item_append(s_info.list, "Location", NULL, NULL, _list_item_selected_cb, (void *)view_location_list_cb);
	elm_list_item_append(s_info.list, "Network", NULL, NULL, _list_item_selected_cb, (void *)view_network_list_cb);
	elm_list_item_append(s_info.list, "OpenGL", NULL, NULL, _list_item_selected_cb, (void *)view_opengl_list_cb);
	elm_list_item_append(s_info.list, "Platform", NULL, NULL, _list_item_selected_cb, (void *)view_platform_list_cb);
	elm_list_item_append(s_info.list, "Screen", NULL, NULL, _list_item_selected_cb, (void *)view_screen_list_cb);
	elm_list_item_append(s_info.list, "Sensors", NULL, NULL, _list_item_selected_cb, (void *)view_sensors_list_cb);
	elm_list_item_append(s_info.list, "Speech", NULL, NULL, _list_item_selected_cb, (void *)view_speech_list_cb);
	elm_list_item_append(s_info.list, "System", NULL, NULL, _list_item_selected_cb, (void *)view_system_list_cb);
	elm_list_item_append(s_info.list, "USB", NULL, NULL, _list_item_selected_cb, (void *)view_usb_list_cb);
	elm_list_item_append(s_info.list, "Vision", NULL, NULL, _list_item_selected_cb, (void *)view_vision_list_cb);
	elm_list_item_append(s_info.list, "Others", NULL, NULL, _list_item_selected_cb, (void *)view_others_list_cb);
}
