/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "$(appName).h"
#include "view_common.h"

#define LIST_BLOCK_SIZE 14

static struct view_info {
	Evas_Object *genlist;
} s_info = {
	.genlist = NULL,
};

static void _item_selected_cb(void *data, Evas_Object *obj, void *event_info);
static Evas_Object *_create_genlist(Evas_Object *parent);
static Elm_Genlist_Item_Class *_create_genlist_item_class(const char *item_style, Elm_Genlist_Item_Text_Get_Cb on_text_get_cb);
static bool _add_items(int index_start, int items_count, Elm_Genlist_Item_Text_Get_Cb on_text_get_cb);

/**
 * @brief Function which creates the main item's sub-view.
 * @param[in] parent The genlist parent object.
 * @param[in] caption The text to be displayed on the top of the view.
 * @param[in] items_count Number of items to be displayed on a genlist.
 * Each item is identified with a number corresponding to the item's index on a genlist.
 * @param[in] on_text_get_cb The callback function invoked on item's redraw. This callback
 * function is assigned to the func.text_get member of the genlist item class structure.
 * @return This function returns 'true' if the sub view was created successfully,
 * otherwise 'false' is returned.
 */
bool view_create_subview(Evas_Object *parent, const char *caption, int items_count, Elm_Genlist_Item_Text_Get_Cb on_text_get_cb)
{
	if (s_info.genlist)
		evas_object_del(s_info.genlist);

	s_info.genlist = _create_genlist(parent);
	if (!s_info.genlist)
		return false;

	if (!_add_items(0, items_count, on_text_get_cb)) {
		evas_object_del(s_info.genlist);
		s_info.genlist = NULL;
		return false;
	}

	elm_naviframe_item_push(parent, caption, NULL, NULL, s_info.genlist, NULL);

	return true;
}

/**
 * @brief Function which creates the group within main item's sub-view.
 * The single text list header is created and genlist items are appended.
 * @param[in] index_start The initial value for items identification.
 * @param[in] items_count Number of items to be displayed on a genlist within created group.
 * Each item is identified with a number corresponding to the item's index on a genlist.
 * @param[in] on_text_get_cb The callback function invoked on item's redraw. This callback
 * function is assigned to the func.text_get member of the genlist item class structure.
 * @return This function returns 'true' if the group was created successfully,
 * otherwise 'false' is returned.
 */
bool view_create_subview_group(int index_start, int items_count, Elm_Genlist_Item_Text_Get_Cb on_text_get_cb)
{
	Elm_Genlist_Item_Class *itc;

	if (!s_info.genlist) {
		dlog_print(DLOG_ERROR, LOG_TAG, "view_subview_append() failed. Genlist is not prior created.");
		return false;
	}

	itc = _create_genlist_item_class("group_index", on_text_get_cb);
	if (!itc)
		return false;

	elm_genlist_item_append(s_info.genlist, itc, (void *)index_start, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
	elm_genlist_item_class_free(itc);

	return _add_items(index_start + 1, items_count - 1, on_text_get_cb);
}

/**
 * @brief Internal callback function invoked on genlist item selection.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _item_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Smart_Cb on_select_cb = (Evas_Smart_Cb)data;
	Elm_Object_Item *item = event_info;

	elm_genlist_item_selected_set(item, EINA_FALSE);

	if (!on_select_cb) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Genlist item selection callback function is not attached.");
		return;
	}

	on_select_cb(NULL, obj, event_info);
}

/**
 * @brief Internal function which creates a genlist for parent object.
 * @param[in] parent The parent object for a genlist.
 * @return This function returns a genlist object if is was successfully created,
 * otherwise NULL is returned.
 */
static Evas_Object *_create_genlist(Evas_Object *parent)
{
	Evas_Object *list = elm_genlist_add(parent);
	if (!list) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_genlist_add() failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(list, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(list, EVAS_HINT_FILL, EVAS_HINT_FILL);

	elm_genlist_block_count_set(list, LIST_BLOCK_SIZE);
	elm_genlist_mode_set(list, ELM_LIST_COMPRESS);
	evas_object_smart_callback_add(list, "selected", _item_selected_cb, NULL);

	evas_object_show(list);

	return list;
}

/**
 * @brief Internal function which creates an item class for genlist object.
 * @param[in] item_style The name of the style to be applied to the item.
 * @param[in] on_text_get_cb on_text_get_cb The callback function invoked on item's redraw.
 * This callback function is assigned to the func.text_get member of the genlist item class structure.
  * @return The function returns item class if it was created successfully,
 * otherwise 'NULL' is returned.
 */
static Elm_Genlist_Item_Class *_create_genlist_item_class(const char *item_style, Elm_Genlist_Item_Text_Get_Cb on_text_get_cb)
{
	Elm_Genlist_Item_Class *itc = elm_genlist_item_class_new();
	if (!itc) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_genlist_item_class_new() failed.");
		return NULL;
	}

	itc->item_style = item_style;
	itc->func.text_get = on_text_get_cb;
	itc->func.content_get = NULL;
	itc->func.state_get = NULL;
	itc->func.del = NULL;

	return itc;
}

/**
 * @brief Internal function which adds items to the genlist.
 * @param[in] index_start The initial value for items identification.
 * @param[in] items_count Number of items to be added to the genlist.
 * Each item is identified with a number corresponding to the its index on a genlist.
 * @param[in] on_text_get_cb The callback function invoked on item's redraw. This callback
 * function is assigned to the func.text_get member of the genlist item class structure.
 * @return This function returns 'true' if the items were created successfully,
 * otherwise 'false' is returned.
 */
static bool _add_items(int index_start, int items_count, Elm_Genlist_Item_Text_Get_Cb on_text_get_cb)
{
	int i;
	Elm_Genlist_Item_Class *itc = _create_genlist_item_class("double_label", on_text_get_cb);

	if (!itc)
		return false;

	for (i = index_start; i < index_start + items_count; i++)
		elm_genlist_item_append(s_info.genlist, itc, (void *)i, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);

	elm_genlist_item_class_free(itc);

	return true;
}
