/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "view_common.h"

#define BLUETOOTH_ITEM_INDEX 0
#define BLUETOOTH_ITEM_COUNT 8
#define NFC_ITEM_INDEX BLUETOOTH_ITEM_INDEX+BLUETOOTH_ITEM_COUNT
#define NFC_ITEM_COUNT 4
#define PUSH_ITEM_INDEX NFC_ITEM_INDEX+NFC_ITEM_COUNT
#define PUSH_ITEM_COUNT 2
#define SECURE_ITEM_INDEX PUSH_ITEM_INDEX+PUSH_ITEM_COUNT
#define SECURE_ITEM_COUNT 4
#define TELEPHONY_ITEM_INDEX SECURE_ITEM_INDEX+SECURE_ITEM_COUNT
#define TELEPHONY_ITEM_COUNT 3
#define TETHERING_ITEM_INDEX TELEPHONY_ITEM_INDEX+TELEPHONY_ITEM_COUNT
#define TETHERING_ITEM_COUNT 5
#define WIFI_ITEM_INDEX TETHERING_ITEM_INDEX+TETHERING_ITEM_COUNT
#define WIFI_ITEM_COUNT 5
#define ITEMS_COUNT BLUETOOTH_ITEM_COUNT+NFC_ITEM_COUNT+PUSH_ITEM_COUNT+SECURE_ITEM_COUNT+\
						TELEPHONY_ITEM_COUNT+TETHERING_ITEM_COUNT+WIFI_ITEM_COUNT

static struct view_info {
	char *description;
	char *feature_key;
} s_info[ITEMS_COUNT] = {
		{"Bluetooth",							NULL},
		{"Bluetooth",							"http://tizen.org/feature/network.bluetooth"},
		{"Handsfree Profile",					"http://tizen.org/feature/network.bluetooth.audio.call"},
		{"Advanced Audio Distribute Profile",	"http://tizen.org/feature/network.bluetooth.audio.media"},
		{"Health Device Profile",				"http://tizen.org/feature/network.bluetooth.health"},
		{"Human Input Device",					"http://tizen.org/feature/network.bluetooth.hid"},
		{"Low Energy",							"http://tizen.org/feature/network.bluetooth.le"},
		{"Object Push Profile",					"http://tizen.org/feature/network.bluetooth.opp"},
		{"NFC",									NULL},
		{"NFC",									"http://tizen.org/feature/network.nfc"},
		{"Card Readers",						"http://tizen.org/feature/network.nfc.card_emulation"},
		{"Reserved Push",						"http://tizen.org/feature/network.nfc.reserved_push"},
		{"PUSH",								NULL},
		{"IP Push Service",						"http://tizen.org/feature/network.push"},
		{"Secure",								NULL},
		{"Secure Elements",						"http://tizen.org/feature/network.secure_element"},
		{"eSe Secure Elements",					"http://tizen.org/feature/network.secure_element.ese"},
		{"UICC Secure Elements",				"http://tizen.org/feature/network.secure_element.uicc"},
		{"Telephony",							NULL},
		{"Telephony",							"http://tizen.org/feature/network.telephony"},
		{"MMS",									"http://tizen.org/feature/network.telephony.mms"},
		{"Tethering",							NULL},
		{"Tethering",							"http://tizen.org/feature/network.tethering"},
		{"Bluetooth Tethering",					"http://tizen.org/feature/network.tethering.bluetooth"},
		{"USB Tethering",						"http://tizen.org/feature/network.tethering.usb"},
		{"Wi-Fi Tethering",						"http://tizen.org/feature/network.tethering.wifi"},
		{"Wi-Fi",								NULL},
		{"Wi-Fi",								"http://tizen.org/feature/network.wifi"},
		{"Wi-Fi Direct",						"http://tizen.org/feature/network.wifi.direct"},
		{"Wi-Fi Direct Display",				"http://tizen.org/feature/network.wifi.direct.display"},
		{"Wi-Fi Direct Service Discovery",		"http://tizen.org/feature/network.wifi.direct.service_discovery"}
};

static char *_item_text_get_cb(void *data, Evas_Object *obj, const char *part);

/**
 * @brief Callback function which creates the 'Network' item sub-view.
 * This callback function is invoked on 'Network' item selection from the main view.
 * @param[in] data The user data passed to the elm_list_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
void view_network_list_cb(void *data, Evas_Object *obj, void *event_info)
{
	if (!view_create_subview((Evas_Object *)data, "Network", 0, _item_text_get_cb))
		return;

	view_create_subview_group(BLUETOOTH_ITEM_INDEX, BLUETOOTH_ITEM_COUNT, _item_text_get_cb);
	view_create_subview_group(NFC_ITEM_INDEX, NFC_ITEM_COUNT, _item_text_get_cb);
	view_create_subview_group(PUSH_ITEM_INDEX, PUSH_ITEM_COUNT, _item_text_get_cb);
	view_create_subview_group(SECURE_ITEM_INDEX, SECURE_ITEM_COUNT, _item_text_get_cb);
	view_create_subview_group(TELEPHONY_ITEM_INDEX, TELEPHONY_ITEM_COUNT, _item_text_get_cb);
	view_create_subview_group(TETHERING_ITEM_INDEX, TETHERING_ITEM_COUNT, _item_text_get_cb);
	view_create_subview_group(WIFI_ITEM_INDEX, WIFI_ITEM_COUNT, _item_text_get_cb);
}

/**
 * @brief Internal callback function invoked on genlist item display.
 * This function is assigned to the func.text_get handler of genlist item's class structure.
 * The item's area consists if two labels:
 * - top label where the feature description is displayed,
 * - bottom label where the feature's status is displayed.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static char *_item_text_get_cb(void *data, Evas_Object *obj, const char *part)
{
	int index = (int)data;
	char buf[MAX_STR] = {0,};
	bool value;

	if (!strcmp(part, "elm.text")) {
		return strdup(s_info[index].description);
	} else if (!strcmp(part, "elm.text.sub")) {
		value = data_get_feature_state(s_info[index].feature_key);
		snprintf(buf, MAX_STR, "%s", value ? "Supported" : "Not Supported");
		return strdup(buf);
	}

	return NULL;
}
