/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "view_common.h"

#define CPU_ITEM_INDEX 0
#define CPU_ITEM_COUNT 4
#define FPU_ITEM_INDEX CPU_ITEM_INDEX+CPU_ITEM_COUNT
#define FPU_ITEM_COUNT 6
#define VERSION_ITEM_INDEX FPU_ITEM_INDEX+FPU_ITEM_COUNT
#define VERSION_ITEM_COUNT 4
#define ITEMS_COUNT CPU_ITEM_COUNT+FPU_ITEM_COUNT+VERSION_ITEM_COUNT

static struct view_info {
	char *description;
	char *feature_key;
} s_info[ITEMS_COUNT] = {
		{"CPU",	NULL},
		{"CPU ARMv6",	"http://tizen.org/feature/platform.core.cpu.arch.armv6"},
		{"CPU ARMv7",	"http://tizen.org/feature/platform.core.cpu.arch.armv7"},
		{"CPU x86",	"http://tizen.org/feature/platform.core.cpu.arch.x86"},
		{"FPU",	NULL},
		{"FPU SSE2",	"http://tizen.org/feature/platform.core.fpu.arch.sse2"},
		{"FPU SSE3",	"http://tizen.org/feature/platform.core.fpu.arch.sse3"},
		{"FPU SSSE3",	"http://tizen.org/feature/platform.core.fpu.arch.ssse3"},
		{"FPU VFPV2",	"http://tizen.org/feature/platform.core.fpu.arch.vfpv2"},
		{"FPU VFPV3",	"http://tizen.org/feature/platform.core.fpu.arch.vfpv3"},
		{"Version",	NULL},
		{"Native API Version",	"http://tizen.org/feature/platform.native.api.version"},
		{"Platform Version",	"http://tizen.org/feature/platform.version"},
		{"Web API Version",	"http://tizen.org/feature/platform.web.api.version"},
};

static char *_item_text_get_cb(void *data, Evas_Object *obj, const char *part);

/**
 * @brief Callback function which creates the 'Platform' item sub-view.
 * This callback function is invoked on 'Platform' item selection from the main view.
 * @param[in] data The user data passed to the elm_list_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
void view_platform_list_cb(void *data, Evas_Object *obj, void *event_info)
{
	if (!view_create_subview((Evas_Object *)data, "Platform", 0, _item_text_get_cb))
		return;

	view_create_subview_group(CPU_ITEM_INDEX, CPU_ITEM_COUNT, _item_text_get_cb);
	view_create_subview_group(FPU_ITEM_INDEX, FPU_ITEM_COUNT, _item_text_get_cb);
	view_create_subview_group(VERSION_ITEM_INDEX, VERSION_ITEM_COUNT, _item_text_get_cb);
}

/**
 * @brief Internal callback function invoked on genlist item display.
 * This function is assigned to the func.text_get handler of genlist item's class structure.
 * The item's area consists if two labels:
 * - top label where the feature description is displayed,
 * - bottom label where the feature's status is displayed.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static char *_item_text_get_cb(void *data, Evas_Object *obj, const char *part)
{
	int index = (int)data;
	char buf[MAX_STR] = {0,};
	bool value;

	if (!strcmp(part, "elm.text")) {
		return strdup(s_info[index].description);
	} else if (!strcmp(part, "elm.text.sub")) {
		if (index < 10) {
			value = data_get_feature_state(s_info[index].feature_key);
			snprintf(buf, MAX_STR, "%s", value ? "Supported" : "Not Supported");
			return strdup(buf);
		} else {
			return data_get_feature_description(s_info[index].feature_key);
		}
	}

	return NULL;
}
